"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var pretur_redux_1 = require('pretur.redux');
var Querier_1 = require('./query/Querier');
var UniqueReducible_1 = require('./UniqueReducible');
var actions_1 = require('./actions');
var Reactive = (function (_super) {
    __extends(Reactive, _super);
    function Reactive(set, query, buildFetcher) {
        if (buildFetcher === void 0) { buildFetcher = null; }
        _super.call(this);
        if (set === null) {
            return;
        }
        this._set = set;
        this._querier = new Querier_1.default(query);
        this._initialized = false;
        this._buildFetcher = buildFetcher;
    }
    Object.defineProperty(Reactive.prototype, "set", {
        get: function () {
            return this._set;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "querier", {
        get: function () {
            return this._querier;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "initialized", {
        get: function () {
            return this._initialized;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "valid", {
        get: function () {
            return this._set.valid;
        },
        enumerable: true,
        configurable: true
    });
    Reactive.prototype.reduce = function (action) {
        var _this = this;
        if (actions_1.CLAY_REACTIVE_REFRESH.is(this.id, action)) {
            pretur_redux_1.emit(action, function (dispatch) {
                var fetcher = _this._buildFetcher();
                _this.forceRefresh(dispatch, fetcher);
                fetcher.fetch();
            });
            return this;
        }
        if (actions_1.CLAY_REACTIVE_SET_INITIALIZED.is(this.id, action)) {
            var clone = this.clone();
            clone._initialized = true;
            return clone;
        }
        var newSet = this._set.reduce(action);
        var newQuerier = this._querier.reduce(action);
        if (newSet !== this._set || newQuerier !== this._querier) {
            var clone_1 = this.clone();
            clone_1._set = newSet;
            clone_1._querier = newQuerier;
            if (this._buildFetcher && newQuerier !== this._querier) {
                pretur_redux_1.emit(action, function (dispatch) {
                    var fetcher = _this._buildFetcher();
                    clone_1.forceRefresh(dispatch, fetcher);
                    fetcher.fetch();
                });
            }
            return clone_1;
        }
        return this;
    };
    Reactive.prototype.init = function (dispatch, fetcher) {
        var _this = this;
        return this._set.refresh(dispatch, this._querier.plain, fetcher).then(function (count) {
            dispatch(actions_1.CLAY_REACTIVE_SET_INITIALIZED.create.unicast(_this.id));
            return count;
        });
    };
    Reactive.prototype.initIfNeeded = function (dispatch, fetcher) {
        if (!this._initialized) {
            this.init(dispatch, fetcher);
        }
    };
    Reactive.prototype.forceRefresh = function (dispatch, fetcher) {
        return this._set.refresh(dispatch, this._querier.plain, fetcher);
    };
    Reactive.prototype.ensureValidity = function (dispatch) {
        if (this.valid) {
            return true;
        }
        dispatch(actions_1.CLAY_DATA_DECAY.create.broadcast());
        return false;
    };
    Reactive.prototype.ensureValidityAndSync = function (dispatch, synchronizer, autoDecay, autoRefresh) {
        var _this = this;
        if (autoDecay === void 0) { autoDecay = true; }
        if (autoRefresh === void 0) { autoRefresh = true; }
        if (this.valid) {
            this._set.appendSynchronizationModels(synchronizer);
            if (autoRefresh) {
                synchronizer.listen().then(function (errors) {
                    if (!errors || !Array.isArray(errors) || errors.length === 0) {
                        dispatch(actions_1.CLAY_REACTIVE_REFRESH.create.unicast(_this.id));
                    }
                });
            }
            return true;
        }
        if (autoDecay) {
            dispatch(actions_1.CLAY_DATA_DECAY.create.broadcast());
        }
        return false;
    };
    Reactive.prototype.cloneOverride = function (clone) {
        _super.prototype.cloneOverride.call(this, clone);
        clone._set = this._set;
        clone._querier = this._querier;
        clone._initialized = this._initialized;
        clone._buildFetcher = this._buildFetcher;
    };
    Reactive.prototype.createInstance = function () {
        return new Reactive(null, null);
    };
    return Reactive;
}(UniqueReducible_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Reactive;
